@@ -28,6 +28,7 @@
 
 #include "goadaemon.h"
 #include "goabackend/goabackend.h"
+#include "goabackend/goaprovider-priv.h"
 #include "goabackend/goautils.h"
 
 struct _GoaDaemon
@@ -210,26 +211,6 @@ on_network_monitor_network_changed (GoaDaemon *self, gboolean available)
   queue_check_credentials (self);
 }
 
-#ifdef GOA_KERBEROS_ENABLED
-static void
-activate_identity_service (GoaDaemon *self)
-{
-  GoaProvider *provider;
-
-  /* We activate the identity service implicitly by using the kerberos
-   * backend.  This way if the kerberos backend isn't enabled, we don't
-   * end up starting the identity service needlessly
-   */
-  provider = goa_provider_get_for_provider_type (GOA_KERBEROS_NAME);
-
-  if (provider != NULL)
-    {
-      g_debug ("activated kerberos provider");
-      g_object_unref (provider);
-    }
-}
-#endif
-
 static void
 goa_daemon_init (GoaDaemon *self)
 {
@@ -243,6 +224,8 @@ goa_daemon_init (GoaDaemon *self)
   goa_error_domain = GOA_ERROR;
   goa_error_domain; /* shut up -Wunused-but-set-variable */
 
+  goa_provider_ensure_builtins_loaded ();
+
   /* TODO: maybe nicer to pass in a GDBusConnection* construct property */
   self->connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
 
@@ -291,10 +274,6 @@ goa_daemon_init (GoaDaemon *self)
   g_dbus_object_manager_server_set_connection (self->object_manager, self->connection);
 
   queue_check_credentials (self);
-
-#ifdef GOA_KERBEROS_ENABLED
-  activate_identity_service (self);
-#endif
 }
 
 static void
@@ -362,6 +341,22 @@ diff_sorted_lists (GList *list1,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static const gchar *
+group_to_id (const gchar *group)
+{
+  g_return_val_if_fail (g_str_has_prefix (group, "Account "), NULL);
+  return group + sizeof "Account " - 1;
+}
+
+static gchar *
+object_path_to_group (const gchar *object_path)
+{
+  g_return_val_if_fail (g_str_has_prefix (object_path, "/org/gnome/OnlineAccounts/Accounts/"), NULL);
+  return g_strdup_printf ("Account %s", object_path + sizeof "/org/gnome/OnlineAccounts/Accounts/" - 1);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 typedef struct
 {
   GKeyFile *key_file;
@@ -417,6 +412,7 @@ add_config_file (GoaDaemon     *self,
     }
   else
     {
+      gboolean needs_update = FALSE;
       gchar **groups;
       const char *guid;
       gsize num_groups;
@@ -447,9 +443,44 @@ add_config_file (GoaDaemon     *self,
                   if (session_id != NULL &&
                       g_strcmp0 (session_id, guid) != 0)
                     {
+                      GoaProvider *provider = NULL;
+                      const gchar *id;
+                      gchar *provider_type = NULL;
+
                       g_debug ("ignoring account \"%s\" in file %s because it's stale",
                                groups[n], path);
+
+                      needs_update = g_key_file_remove_group (key_file, groups[n], NULL);
+
+                      id = group_to_id (groups[n]);
+                      if (id == NULL)
+                        {
+                          g_warning ("Unable to get account ID from group: %s", groups[n]);
+                          goto cleanup_and_continue;
+                        }
+
+                      provider_type = g_key_file_get_string (key_file, groups[n], "Provider", NULL);
+                      if (provider_type != NULL)
+                        provider = goa_provider_get_for_provider_type (provider_type);
+
+                      if (provider == NULL)
+                        {
+                          g_warning ("Unsupported account type %s for ID %s (no provider)", provider_type, id);
+                          goto cleanup_and_continue;
+                        }
+
+                      error = NULL;
+                      if (!goa_utils_delete_credentials_for_id_sync (provider, id, NULL, &error))
+                        {
+                          g_warning ("Unable to clean-up stale keyring entries: %s", error->message);
+                          g_error_free (error);
+                          goto cleanup_and_continue;
+                        }
+
+                    cleanup_and_continue:
+                      g_clear_object (&provider);
                       g_free (groups[n]);
+                      g_free (provider_type);
                       g_free (session_id);
                       continue;
                     }
@@ -457,7 +488,7 @@ add_config_file (GoaDaemon     *self,
                 }
               else
                 {
-                  g_key_file_remove_key (key_file, groups[n], "SessionId", NULL);
+                  needs_update = g_key_file_remove_key (key_file, groups[n], "SessionId", NULL);
                 }
 
               g_hash_table_insert (group_name_to_key_file_data,
@@ -472,6 +503,17 @@ add_config_file (GoaDaemon     *self,
         }
       g_free (groups);
 
+      if (needs_update)
+        {
+          error = NULL;
+          if (!g_key_file_save_to_file (key_file, path, &error))
+            {
+              g_prefix_error (&error, "Error writing key-value-file %s: ", path);
+              g_warning ("%s (%s, %d)", error->message, g_quark_to_string (error->domain), error->code);
+              g_error_free (error);
+            }
+        }
+
       *key_files_to_free = g_list_prepend (*key_files_to_free, key_file);
     }
 }
@@ -490,6 +532,7 @@ update_account_object (GoaDaemon           *self,
   GoaAccount *account;
   GoaProvider *provider;
   gboolean is_locked;
+  gboolean is_temporary;
   gboolean ret;
   gchar *identity;
   gchar *presentation_identity;
@@ -520,6 +563,7 @@ update_account_object (GoaDaemon           *self,
   identity = g_key_file_get_string (key_file, group, "Identity", NULL);
   presentation_identity = g_key_file_get_string (key_file, group, "PresentationIdentity", NULL);
   is_locked = g_key_file_get_boolean (key_file, group, "IsLocked", NULL);
+  is_temporary = g_key_file_get_boolean (key_file, group, "IsTemporary", NULL);
   if (just_added)
     {
       account = goa_account_skeleton_new ();
@@ -542,6 +586,7 @@ update_account_object (GoaDaemon           *self,
   goa_account_set_identity (account, identity);
   goa_account_set_presentation_identity (account, presentation_identity);
   goa_account_set_is_locked (account, is_locked);
+  goa_account_set_is_temporary (account, is_temporary);
 
   error = NULL;
   if (!goa_provider_build_object (provider, object, key_file, group, self->connection, just_added, &error))
@@ -573,19 +618,12 @@ update_account_object (GoaDaemon           *self,
   return ret;
 }
 
-static gchar *
-object_path_to_group (const gchar *object_path)
-{
-  g_return_val_if_fail (g_str_has_prefix (object_path, "/org/gnome/OnlineAccounts/Accounts/"), NULL);
-  return g_strdup_printf ("Account %s", object_path + sizeof "/org/gnome/OnlineAccounts/Accounts/" - 1);
-}
-
 static void
 process_config_entries (GoaDaemon  *self,
                         GHashTable *group_name_to_key_file_data)
 {
   GHashTableIter iter;
-  const gchar *id;
+  const gchar *group;
   KeyFileData *key_file_data;
   GList *existing_object_paths;
   GList *config_object_paths;
@@ -611,15 +649,18 @@ process_config_entries (GoaDaemon  *self,
 
   config_object_paths = NULL;
   g_hash_table_iter_init (&iter, group_name_to_key_file_data);
-  while (g_hash_table_iter_next (&iter, (gpointer*) &id, (gpointer*) &key_file_data))
+  while (g_hash_table_iter_next (&iter, (gpointer*) &group, (gpointer*) &key_file_data))
     {
+      const gchar *id;
       gchar *object_path;
 
+      id = group_to_id (group);
+
       /* create and validate object path */
-      object_path = g_strdup_printf ("/org/gnome/OnlineAccounts/Accounts/%s", id + sizeof "Account " - 1);
-      if (strstr (id + sizeof "Account " - 1, "/") != NULL || !g_variant_is_object_path (object_path))
+      object_path = g_strdup_printf ("/org/gnome/OnlineAccounts/Accounts/%s", id);
+      if (strstr (id, "/") != NULL || !g_variant_is_object_path (object_path))
         {
-          g_warning ("`%s' is not a valid account identifier", id);
+          g_warning ("`%s' is not a valid account identifier", group);
           g_free (object_path);
           continue;
         }
@@ -755,6 +796,8 @@ goa_daemon_reload_configuration (GoaDaemon *self)
   g_list_free_full (key_files_to_free, (GDestroyNotify) g_key_file_free);
 }
 
+/* ---------------------------------------------------------------------------------------------------- */
+
 static gchar *
 generate_new_id (GoaDaemon *self)
 {
@@ -800,6 +843,7 @@ get_all_providers_cb (GObject      *source,
   gchar *group;
   gchar *key_file_data;
   gsize length;
+  gsize n_credentials;
   gchar *object_path;
   GVariantIter iter;
   const gchar *key;
@@ -911,37 +955,26 @@ get_all_providers_cb (GObject      *source,
       g_key_file_set_string (key_file, group, key, value);
     }
 
-  g_free (key_file_data);
   error = NULL;
-  key_file_data = g_key_file_to_data (key_file,
-                                      &length,
-                                      &error);
-  if (key_file_data == NULL)
-    {
-      g_prefix_error (&error, "Error generating key-value-file: ");
-      g_dbus_method_invocation_take_error (data->invocation, error);
-      goto out;
-    }
-
-  error = NULL;
-  if (!g_file_set_contents (path,
-                            key_file_data,
-                            length,
-                            &error))
+  if (!g_key_file_save_to_file (key_file, path, &error))
     {
       g_prefix_error (&error, "Error writing key-value-file %s: ", path);
       g_dbus_method_invocation_take_error (data->invocation, error);
       goto out;
     }
 
-  /* We don't want to fail AddAccount if we could not store the
-   * credentials in the keyring.
-   */
-  goa_utils_store_credentials_for_id_sync (provider,
-                                           id,
-                                           data->credentials,
-                                           NULL, /* GCancellable */
-                                           NULL);
+  n_credentials = g_variant_n_children (data->credentials);
+  if (n_credentials > 0)
+    {
+      /* We don't want to fail AddAccount if we could not store the
+       * credentials in the keyring.
+       */
+      goa_utils_store_credentials_for_id_sync (provider,
+                                               id,
+                                               data->credentials,
+                                               NULL, /* GCancellable */
+                                               NULL);
+    }
 
   goa_daemon_reload_configuration (data->daemon);
 
@@ -997,19 +1030,80 @@ on_manager_handle_add_account (GoaManager             *manager,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+typedef struct
+{
+  GoaObject *object;
+  GList *invocations;
+} ObjectInvocationData;
+
+static ObjectInvocationData *
+object_invocation_data_new (GoaObject             *object,
+                 GDBusMethodInvocation *invocation)
+{
+  ObjectInvocationData *data;
+  data = g_slice_new0 (ObjectInvocationData);
+  data->object = g_object_ref (object);
+  data->invocations = g_list_prepend (data->invocations, invocation);
+  return data;
+}
+
+static void
+object_invocation_data_unref (ObjectInvocationData *data)
+{
+  g_list_free (data->invocations);
+  g_object_unref (data->object);
+  g_slice_free (ObjectInvocationData, data);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+static void
+remove_account_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GTask *task = G_TASK (user_data);
+  GoaDaemon *self;
+  GDBusMethodInvocation *invocation;
+  GError *error;
+  GoaAccount *account;
+  GoaProvider *provider = GOA_PROVIDER (source_object);
+  ObjectInvocationData *data;
+
+  self = GOA_DAEMON (g_task_get_source_object (task));
+  data = g_task_get_task_data (task);
+
+  error= NULL;
+  if (!goa_provider_remove_account_finish (provider, res, &error))
+    {
+      g_warning ("goa_provider_remove_account() failed: %s (%s, %d)",
+                 error->message,
+                 g_quark_to_string (error->domain),
+                 error->code);
+      g_error_free (error);
+    }
+
+  goa_daemon_reload_configuration (self);
+
+  account = goa_object_peek_account (data->object);
+  invocation = G_DBUS_METHOD_INVOCATION (data->invocations->data);
+  goa_account_complete_remove (account, invocation);
+
+  g_object_unref (task);
+}
+
 static gboolean
 on_account_handle_remove (GoaAccount            *account,
                           GDBusMethodInvocation *invocation,
                           gpointer               user_data)
 {
   GoaDaemon *self = GOA_DAEMON (user_data);
+  GoaObject *object;
   GoaProvider *provider;
   GKeyFile *key_file;
+  GTask *task;
+  ObjectInvocationData *data;
   const gchar *provider_type;
   gchar *path;
   gchar *group;
-  gchar *data;
-  gsize length;
   GError *error;
 
   provider = NULL;
@@ -1017,7 +1111,7 @@ on_account_handle_remove (GoaAccount            *account,
   path = NULL;
   group = NULL;
   key_file = NULL;
-  data = NULL;
+  task = NULL;
 
   if (goa_account_get_is_locked (account))
     {
@@ -1057,21 +1151,7 @@ on_account_handle_remove (GoaAccount            *account,
     }
 
   error = NULL;
-  data = g_key_file_to_data (key_file,
-                             &length,
-                             &error);
-  if (data == NULL)
-    {
-      g_prefix_error (&error, "Error generating key-value-file: ");
-      g_dbus_method_invocation_take_error (invocation, error);
-      goto out;
-    }
-
-  error = NULL;
-  if (!g_file_set_contents (path,
-                            data,
-                            length,
-                            &error))
+  if (!g_key_file_save_to_file (key_file, path, &error))
     {
       g_prefix_error (&error, "Error writing key-value-file %s: ", path);
       g_dbus_method_invocation_take_error (invocation, error);
@@ -1104,19 +1184,23 @@ on_account_handle_remove (GoaAccount            *account,
     }
 
   error = NULL;
-  if (!goa_utils_delete_credentials_sync (provider, account, NULL, &error))
+  if (!goa_utils_delete_credentials_for_account_sync (provider, account, NULL, &error))
     {
       g_dbus_method_invocation_take_error (invocation, error);
       goto out;
     }
 
-  goa_daemon_reload_configuration (self);
+  object = GOA_OBJECT (g_dbus_interface_get_object (G_DBUS_INTERFACE (account)));
+  data = object_invocation_data_new (object, invocation);
 
-  goa_account_complete_remove (account, invocation);
+  task = g_task_new (self, NULL, NULL, NULL);
+  g_task_set_task_data (task, data, (GDestroyNotify) object_invocation_data_unref);
+
+  goa_provider_remove_account (provider, object, NULL, remove_account_cb, g_object_ref (task));
 
  out:
   g_clear_object (&provider);
-  g_free (data);
+  g_clear_object (&task);
   g_clear_pointer (&key_file, (GDestroyNotify) g_key_file_free);
   g_free (group);
   g_free (path);
@@ -1125,31 +1209,6 @@ on_account_handle_remove (GoaAccount            *account,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-typedef struct
-{
-  GoaObject *object;
-  GList *invocations;
-} EnsureData;
-
-static EnsureData *
-ensure_data_new (GoaObject             *object,
-                 GDBusMethodInvocation *invocation)
-{
-  EnsureData *data;
-  data = g_slice_new0 (EnsureData);
-  data->object = g_object_ref (object);
-  data->invocations = g_list_prepend (data->invocations, invocation);
-  return data;
-}
-
-static void
-ensure_data_unref (EnsureData *data)
-{
-  g_list_free (data->invocations);
-  g_object_unref (data->object);
-  g_slice_free (EnsureData, data);
-}
-
 static gboolean
 is_authorization_error (GError *error)
 {
@@ -1203,10 +1262,10 @@ ensure_credentials_queue_collector (GObject *source_object, GAsyncResult *res, g
 {
   GTask *task = G_TASK (user_data);
   GoaDaemon *self;
-  EnsureData *data;
   GoaAccount *account;
   GoaProvider *provider = GOA_PROVIDER (source_object);
   GError *error;
+  ObjectInvocationData *data;
   gint expires_in;
 
   self = GOA_DAEMON (g_task_get_source_object (task));
@@ -1270,10 +1329,10 @@ ensure_credentials_queue_sort (gconstpointer a, gconstpointer b, gpointer user_d
 static void
 ensure_credentials_queue_check (GoaDaemon *self)
 {
-  EnsureData *data;
   GoaAccount *account;
   GoaProvider *provider = NULL;
   GTask *task;
+  ObjectInvocationData *data;
   const gchar *id;
   const gchar *provider_type;
   gint64 timestamp;
@@ -1289,7 +1348,7 @@ ensure_credentials_queue_check (GoaDaemon *self)
   task = G_TASK (g_queue_peek_head (self->ensure_credentials_queue));
   self->ensure_credentials_running = TRUE;
 
-  data = (EnsureData *) g_task_get_task_data (task);
+  data = (ObjectInvocationData *) g_task_get_task_data (task);
   account = goa_object_peek_account (data->object);
 
   id = goa_account_get_id (account);
@@ -1328,7 +1387,7 @@ ensure_credentials_queue_coalesce (GoaDaemon *self, GoaObject *object, GDBusMeth
     {
       GoaAccount *account_queued;
       GTask *task = G_TASK (l->data);
-      EnsureData *data;
+      ObjectInvocationData *data;
       const gchar *id_queued;
 
       data = g_task_get_task_data (task);
@@ -1358,10 +1417,10 @@ on_account_handle_ensure_credentials (GoaAccount            *account,
                                       gpointer               user_data)
 {
   GoaDaemon *self = GOA_DAEMON (user_data);
-  EnsureData *data;
   GoaObject *object;
   GoaProvider *provider = NULL;
   GTask *task = NULL;
+  ObjectInvocationData *data;
   const gchar *id;
   const gchar *method_name;
   const gchar *provider_type;
@@ -1397,11 +1456,11 @@ on_account_handle_ensure_credentials (GoaAccount            *account,
       goto out;
     }
 
-  data = ensure_data_new (object, invocation);
+  data = object_invocation_data_new (object, invocation);
 
   task = g_task_new (self, NULL, NULL, NULL);
   g_task_set_priority (task, G_PRIORITY_HIGH);
-  g_task_set_task_data (task, data, (GDestroyNotify) ensure_data_unref);
+  g_task_set_task_data (task, data, (GDestroyNotify) object_invocation_data_unref);
   g_queue_push_tail (self->ensure_credentials_queue, g_object_ref (task));
 
   ensure_credentials_queue_check (self);
@@ -1429,11 +1488,11 @@ goa_daemon_check_credentials (GoaDaemon *self)
   objects = g_dbus_object_manager_get_objects (G_DBUS_OBJECT_MANAGER (self->object_manager));
   for (l = objects; l != NULL; l = l->next)
     {
-      EnsureData *data;
       GoaAccount *account;
       GoaObject *object = GOA_OBJECT (l->data);
       GoaProvider *provider = NULL;
       GTask *task = NULL;
+      ObjectInvocationData *data;
       const gchar *id;
       const gchar *provider_type;
       gint64 timestamp;
@@ -1465,11 +1524,11 @@ goa_daemon_check_credentials (GoaDaemon *self)
           continue;
         }
 
-      data = ensure_data_new (object, NULL);
+      data = object_invocation_data_new (object, NULL);
 
       task = g_task_new (self, NULL, NULL, NULL);
       g_task_set_priority (task, G_PRIORITY_LOW);
-      g_task_set_task_data (task, data, (GDestroyNotify) ensure_data_unref);
+      g_task_set_task_data (task, data, (GDestroyNotify) object_invocation_data_unref);
       g_queue_push_tail (self->ensure_credentials_queue, g_object_ref (task));
 
       g_clear_object (&provider);
