@@ -18,14 +18,20 @@
 
 #include "config.h"
 
-#include <glib/gi18n.h>
 #include <glib-unix.h>
 
+#include <locale.h>
 #include <signal.h>
 #include <gio/gio.h>
+#include <libintl.h>
 
 #include "goadaemon.h"
 
+#ifdef GOA_TELEPATHY_ENABLED
+#include "goatpaccountlinker.h"
+#endif
+
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 static GMainLoop *loop = NULL;
@@ -39,6 +45,10 @@ static GOptionEntry opt_entries[] =
 };
 static GoaDaemon *the_daemon = NULL;
 
+#ifdef GOA_TELEPATHY_ENABLED
+static GoaTpAccountLinker *tp_linker = NULL;
+#endif
+
 static void
 on_bus_acquired (GDBusConnection *connection,
                  const gchar     *name,
@@ -64,6 +74,10 @@ on_name_acquired (GDBusConnection *connection,
                   gpointer         user_data)
 {
   g_debug ("Acquired the name %s on the session message bus", name);
+
+#ifdef GOA_TELEPATHY_ENABLED
+  tp_linker = goa_tp_account_linker_new ();
+#endif
 }
 
 static gboolean
@@ -88,6 +102,11 @@ main (int    argc,
   opt_context = NULL;
   name_owner_id = 0;
 
+  setlocale (LC_ALL, "");
+  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+  textdomain (GETTEXT_PACKAGE);
+
   opt_context = g_option_context_new ("GNOME Online Accounts daemon");
   g_option_context_add_main_entries (opt_context, opt_entries, NULL);
   error = NULL;
@@ -125,6 +144,9 @@ main (int    argc,
 
  out:
   g_clear_object (&the_daemon);
+#ifdef GOA_TELEPATHY_ENABLED
+  g_clear_object (&tp_linker);
+#endif
   if (name_owner_id != 0)
     g_bus_unown_name (name_owner_id);
   g_clear_pointer (&loop, (GDestroyNotify) g_main_loop_unref);
