@@ -81,6 +81,17 @@ static guint goa_provider_get_credentials_generation_real (GoaProvider *self);
 
 static GIcon *goa_provider_get_provider_icon_real (GoaProvider *self,
                                                    GoaObject   *object);
+
+static void goa_provider_remove_account_real (GoaProvider          *self,
+                                              GoaObject            *object,
+                                              GCancellable         *cancellable,
+                                              GAsyncReadyCallback   callback,
+                                              gpointer              user_data);
+
+static gboolean goa_provider_remove_account_finish_real (GoaProvider   *self,
+                                                         GAsyncResult  *res,
+                                                         GError       **error);
+
 static void goa_provider_show_account_real (GoaProvider         *provider,
                                             GoaClient           *client,
                                             GoaObject           *object,
@@ -92,6 +103,82 @@ static void goa_provider_show_account_real (GoaProvider         *provider,
 
 G_DEFINE_ABSTRACT_TYPE (GoaProvider, goa_provider, G_TYPE_OBJECT);
 
+static struct {
+  GoaProviderFeatures feature;
+  const gchar *property;
+  const gchar *blurb;
+} provider_features_info[] = {
+  /* The order in which the features are listed is
+   * important because it affects the order in which they are
+   * displayed in the show_account() UI
+   */
+  {
+    .feature = GOA_PROVIDER_FEATURE_MAIL,
+    .property = "mail-disabled",
+    .blurb = N_("_Mail"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_CALENDAR,
+    .property = "calendar-disabled",
+    .blurb = N_("Cale_ndar"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_CONTACTS,
+    .property = "contacts-disabled",
+    .blurb = N_("_Contacts"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_CHAT,
+    .property = "chat-disabled",
+    .blurb = N_("C_hat"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_DOCUMENTS,
+    .property = "documents-disabled",
+    .blurb = N_("_Documents"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_MUSIC,
+    .property = "music-disabled",
+    .blurb = N_("M_usic"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_PHOTOS,
+    .property = "photos-disabled",
+    .blurb = N_("_Photos"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_FILES,
+    .property = "files-disabled",
+    .blurb = N_("_Files"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_TICKETING,
+    .property = "ticketing-disabled",
+    .blurb = N_("Network _Resources"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_READ_LATER,
+    .property = "read-later-disabled",
+    .blurb = N_("_Read Later"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_PRINTERS,
+    .property = "printers-disabled",
+    .blurb = N_("Prin_ters"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_MAPS,
+    .property = "maps-disabled",
+    .blurb = N_("_Maps"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_INVALID,
+    .property = NULL,
+    .blurb = NULL,
+  }
+};
+
 static void
 goa_provider_get_property (GObject *object,
                            guint property_id,
@@ -159,6 +246,8 @@ goa_provider_class_init (GoaProviderClass *klass)
   klass->ensure_credentials_sync = goa_provider_ensure_credentials_sync_real;
   klass->get_credentials_generation = goa_provider_get_credentials_generation_real;
   klass->get_provider_icon = goa_provider_get_provider_icon_real;
+  klass->remove_account = goa_provider_remove_account_real;
+  klass->remove_account_finish = goa_provider_remove_account_finish_real;
   klass->show_account = goa_provider_show_account_real;
 
 /**
@@ -464,82 +553,6 @@ goa_provider_show_account (GoaProvider         *self,
   GOA_PROVIDER_GET_CLASS (self)->show_account (self, client, object, vbox, grid, dummy);
 }
 
-static struct {
-  GoaProviderFeatures feature;
-  const gchar *property;
-  const gchar *blurb;
-} show_account_items[] = {
-  /* The order in which the features are listed is
-   * important because it affects the order in which they are
-   * displayed in the show_account() UI
-   */
-  {
-    .feature = GOA_PROVIDER_FEATURE_MAIL,
-    .property = "mail-disabled",
-    .blurb = N_("_Mail"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_CALENDAR,
-    .property = "calendar-disabled",
-    .blurb = N_("Cale_ndar"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_CONTACTS,
-    .property = "contacts-disabled",
-    .blurb = N_("_Contacts"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_CHAT,
-    .property = "chat-disabled",
-    .blurb = N_("C_hat"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_DOCUMENTS,
-    .property = "documents-disabled",
-    .blurb = N_("_Documents"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_MUSIC,
-    .property = "music-disabled",
-    .blurb = N_("M_usic"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_PHOTOS,
-    .property = "photos-disabled",
-    .blurb = N_("_Photos"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_FILES,
-    .property = "files-disabled",
-    .blurb = N_("_Files"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_TICKETING,
-    .property = "ticketing-disabled",
-    .blurb = N_("Network _Resources"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_READ_LATER,
-    .property = "read-later-disabled",
-    .blurb = N_("_Read Later"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_PRINTERS,
-    .property = "printers-disabled",
-    .blurb = N_("Prin_ters"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_MAPS,
-    .property = "maps-disabled",
-    .blurb = N_("_Maps"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_INVALID,
-    .property = NULL,
-    .blurb = NULL,
-  }
-};
-
 static void
 goa_provider_show_account_real (GoaProvider         *provider,
                                 GoaClient           *client,
@@ -562,14 +575,14 @@ goa_provider_show_account_real (GoaProvider         *provider,
    * options switches. For example: “Use for Mail”. */
   label = _("Use for");
 
-  for (i = 0; show_account_items[i].property != NULL; i++)
+  for (i = 0; provider_features_info[i].property != NULL; i++)
     {
-      if ((features & show_account_items[i].feature) != 0)
+      if ((features & provider_features_info[i].feature) != 0)
         {
           goa_util_add_row_switch_from_keyfile_with_blurb (grid, row++, object,
                                                            label,
-                                                           show_account_items[i].property,
-                                                           _(show_account_items[i].blurb));
+                                                           provider_features_info[i].property,
+                                                           _(provider_features_info[i].blurb));
           label = NULL;
         }
     }
@@ -737,7 +750,7 @@ goa_provider_ensure_credentials_finish (GoaProvider         *self,
                                         GAsyncResult        *res,
                                         GError             **error)
 {
-  GTask *task = G_TASK (res);
+  GTask *task;
   gboolean had_error;
 
   gboolean ret;
@@ -746,9 +759,11 @@ goa_provider_ensure_credentials_finish (GoaProvider         *self,
   ret = FALSE;
 
   g_return_val_if_fail (GOA_IS_PROVIDER (self), FALSE);
-  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
+  task = G_TASK (res);
+
   g_warn_if_fail (g_task_get_source_tag (task) == goa_provider_ensure_credentials);
 
   /* Workaround for bgo#764163 */
@@ -787,10 +802,46 @@ goa_provider_ensure_credentials_sync (GoaProvider     *self,
                                       GCancellable    *cancellable,
                                       GError         **error)
 {
+  GoaAccount *account = NULL;
+  GoaProviderFeatures features;
+  gboolean disabled = TRUE;
+  gboolean ret = FALSE;
+  guint i;
+
   g_return_val_if_fail (GOA_IS_PROVIDER (self), FALSE);
+  g_return_val_if_fail (GOA_IS_OBJECT (object), FALSE);
   g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-  return GOA_PROVIDER_GET_CLASS (self)->ensure_credentials_sync (self, object, out_expires_in, cancellable, error);
+
+  account = goa_object_get_account (object);
+  g_return_val_if_fail (GOA_IS_ACCOUNT (account), FALSE);
+
+  features = goa_provider_get_provider_features (self);
+
+  for (i = 0; provider_features_info[i].property != NULL; i++)
+    {
+      if ((features & provider_features_info[i].feature) != 0)
+        {
+          gboolean feature_disabled;
+
+          g_object_get (account, provider_features_info[i].property, &feature_disabled, NULL);
+          disabled = disabled && feature_disabled;
+          if (!disabled)
+            break;
+        }
+    }
+
+  if (disabled)
+    {
+      g_set_error_literal (error, GOA_ERROR, GOA_ERROR_NOT_SUPPORTED, _("Account is disabled"));
+      goto out;
+    }
+
+  ret = GOA_PROVIDER_GET_CLASS (self)->ensure_credentials_sync (self, object, out_expires_in, cancellable, error);
+
+ out:
+  g_clear_object (&account);
+  return ret;
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -931,8 +982,8 @@ static struct
   { NULL, NULL }
 };
 
-static void
-ensure_builtins_loaded (void)
+void
+goa_provider_ensure_builtins_loaded (void)
 {
   static gsize once_init_value = 0;
 
@@ -1021,7 +1072,7 @@ goa_provider_get_for_provider_type (const gchar *provider_type)
 
   g_return_val_if_fail (provider_type != NULL, NULL);
 
-  ensure_builtins_loaded ();
+  goa_provider_ensure_builtins_loaded ();
 
   ret = NULL;
 
@@ -1171,7 +1222,7 @@ goa_provider_get_all (GAsyncReadyCallback callback,
   GetAllData *data;
   gint i;
 
-  ensure_builtins_loaded ();
+  goa_provider_ensure_builtins_loaded ();
 
   data = g_slice_new0 (GetAllData);
   data->task = g_task_new (NULL, NULL, callback, user_data);
@@ -1186,8 +1237,8 @@ goa_provider_get_all (GAsyncReadyCallback callback,
     {
       GIOExtension *extension = l->data;
       /* The extensions are loaded in the reverse order we used in
-       * ensure_builtins_loaded, so we need to push extension if front of
-       * the already loaded ones. */
+       * goa_provider_ensure_builtins_loaded, so we need to push
+       * extension if front of the already loaded ones. */
       g_queue_push_head (&data->ret, g_object_new (g_io_extension_get_type (extension), NULL));
     }
 
@@ -1224,14 +1275,17 @@ goa_provider_get_all_finish (GList        **out_providers,
                              GAsyncResult  *result,
                              GError       **error)
 {
-  GTask *task = G_TASK (result);
+  GTask *task;
   GList *providers;
   gboolean had_error;
 
-  g_return_val_if_fail (g_task_is_valid (result, NULL), FALSE);
-  g_return_val_if_fail (g_task_get_source_tag (task) == goa_provider_get_all, FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
+  g_return_val_if_fail (g_task_is_valid (result, NULL), FALSE);
+  task = G_TASK (result);
+
+  g_return_val_if_fail (g_task_get_source_tag (task) == goa_provider_get_all, FALSE);
+
   /* Workaround for bgo#764163 */
   had_error = g_task_had_error (task);
   providers = g_task_propagate_pointer (task, error);
@@ -1250,6 +1304,66 @@ goa_provider_get_all_finish (GList        **out_providers,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+void
+goa_provider_remove_account (GoaProvider          *self,
+                             GoaObject            *object,
+                             GCancellable         *cancellable,
+                             GAsyncReadyCallback   callback,
+                             gpointer              user_data)
+{
+  g_return_if_fail (GOA_IS_PROVIDER (self));
+  g_return_if_fail (GOA_IS_OBJECT (object));
+  g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+
+  GOA_PROVIDER_GET_CLASS (self)->remove_account (self, object, cancellable, callback, user_data);
+}
+
+gboolean
+goa_provider_remove_account_finish (GoaProvider   *self,
+                                    GAsyncResult  *res,
+                                    GError       **error)
+{
+  g_return_val_if_fail (GOA_IS_PROVIDER (self), FALSE);
+  g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  return GOA_PROVIDER_GET_CLASS (self)->remove_account_finish (self, res, error);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+static void
+goa_provider_remove_account_real (GoaProvider          *self,
+                                  GoaObject            *object,
+                                  GCancellable         *cancellable,
+                                  GAsyncReadyCallback   callback,
+                                  gpointer              user_data)
+{
+  GTask *task;
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, goa_provider_remove_account_real);
+  g_task_return_boolean (task, TRUE);
+  g_object_unref (task);
+}
+
+static gboolean
+goa_provider_remove_account_finish_real (GoaProvider   *self,
+                                         GAsyncResult  *res,
+                                         GError       **error)
+{
+  GTask *task;
+
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
+  task = G_TASK (res);
+
+  g_warn_if_fail (g_task_get_source_tag (task) == goa_provider_remove_account_real);
+
+  return g_task_propagate_boolean (task, error);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 /**
  * goa_provider_set_preseed_data:
  * @self: The #GoaProvider
